/*
* Project name:
    mmbSprites.vtft
* Created by:
  Andrew Hazelden
* Generated by:
    Visual TFT
* Date of creation
    10/5/2014
* Time of creation
    9:14:55 AM
* Test configuration:
    MCU:               P32MX460F512L
    Dev.Board:      MikroMMB_for_PIC32_hw_rev_1.10_9A
    Oscillator:        80000000 Hz
    SW:                mikroC PRO for PIC32
                          http://www.mikroe.com/mikroc/pic32/
    Controller:       Mikromedia GAMING Shield
                          http://www.mikroe.com/mikromedia/shields/gaming/

                      
    MikroC Libraries:  

    VisualTFT
    http://www.mikroe.com/eng/products/view/627/visual-tft/

    TFT Graphics by Bitwise (required download)
    http://www.libstock.com/projects/view/914/tft-graphics-library
    
    Andrew Hazelden's Mikromedia Gaming Shield Library (included with the current source code)
    http://www.libstock.com/projects/view/389/andrew-hazelden-s-mikromedia-gaming-shield-library 
      
    The MMB 3D Sprites demo code is based upon the classic Metal Basic example "TriCube" by Ivan Freyman.
    
*/

#include "mmbSprites_objects.h"
#include "gaming-shield.h"
#include "thick_ball_resources.h"
#include "_bwo_lib.h"

#define SCREEN_WIDTH 320
#define SCREEN_HEIGHT 240
#define BALL_SPRITE_WIDTH 24
#define BALL_CREATION_RANGE 6
#define MAX_BALLS 2000

#define radToDegrees 57.296

extern void InitGameShield();
void Clear_Balls();
void Reset_Values();
void Generate_Balls();
void Check_GS();
void Display_Debug_Text();

// Track the number of frames rendered
long frame_counter = 0;

// On-screen drawing location
PointF temp_pos;

// Global vars
float amp = 0.5;
float speed = 0.4;
float axi = 0;
float ayi = 0.01;
float azi = 0.02;
float freq = 0.6;
float phase = 0;

float ax = 0;     //Angle of tilt on X axis in radians
float ay = 0;     //Angle of tilt on Y axis in radians
float az = -3.7; //Angle of tilt on Z axis in radians

int xi = 0;
int yi = 0;
int zi = 0;

float x = 0;
float y = 0;
float z = 0;

float plotX = 0;
float plotY = 0;

int balls=0;
int shapeNum = 1;
float bx[MAX_BALLS];
float by[MAX_BALLS];
float bz[MAX_BALLS];

int displayDebug = 0;
int prevDisplayDebug = 0;

void main() {

  float size = 20;
  float perspective = 0.04;
  
  float xoff = SCREEN_WIDTH/2;
  float yoff = SCREEN_HEIGHT/2;

  int n = 0;

  float dd = 0;
  float xp = 0;
  float yp = 0;
  float zp = 0;
  
  // Setup the MMB Board
  Start_TP();
  
  // Setup the gameshield inputs and outputs
  InitGameShield();

  // Draw a ball
  temp_pos.X = 150;
  temp_pos.Y = 30;
  image_bwi(temp_pos, thick_ball_bwi);

  // Draw the program info
  TFT_Set_Font(TFT_defaultFont, CL_WHITE, FO_HORIZONTAL);
  TFT_Write_Text("MMB  3D  Sprites", 110, 60);
  TFT_Write_Text("by  Andrew  Hazelden", 90, 80);
  TFT_Write_Text("Based  Upon  the  Metal  Basic  Sprite  Demo", 25, 100);
  
  TFT_Write_Text("Tap  the  screen  to  change the  model.", 0, 130);
  
  TFT_Write_Text("Gaming  Shield  Controls:", 0, 160);
  TFT_Write_Text("Press  the  START  button  to  change  the  model.", 0, 180);
  TFT_Write_Text("Press  the  CIRCLE  button  to  show  coordinates.", 0, 200);
  TFT_Write_Text("Use  the  ARROW/X/Triangle  buttons  for  rotation.", 0, 220);

  // Add a 14 second delay for the main titles
  delay_ms(14000);

  // Clear the screen
  TFT_Fill_Screen(CL_BLACK);
  
  // Create the balls shape
  Clear_Balls();
  Generate_Balls();
  
  // Reset the screen rotation values
  Reset_Values();
  
  // Main Loop
  while(1){
    Check_GS(); //Check for Game Shield and screen input
    
    frame_counter++;  //Increment the frame counter
    
    ax += axi;
    ay += ayi;
    az += azi;
    
    phase -= speed;
    
    for(n=0; n<(balls); n++){
      x=bx[n];
      y=by[n];
      z=bz[n];
      
      xp = x * cos(az) - y * sin(az);
      yp = x * sin(az) + y * cos(az);
      
      // The dd variable is for temporary storage
      dd = yp * cos(ax) - z * sin(ax);   
      zp = yp * sin(ax) + z * cos(ax);
      yp = dd;
      
      dd = xp * cos(ay) + zp * sin(ay);
      zp = -xp * sin(ay) + zp * cos(ay);
      xp = dd;
      
      // Calculate the perspective effects
      plotX = (xp / (1 + zp * perspective)) * size + xoff;
      plotY = (yp / (1 + zp * perspective)) * size + yoff;
      
      if(plotX < 0) plotX = 0;
      if(plotY < 0) plotY = 0;
      
      // Plot the ball if it is on screen
      if( (plotX > 0) && (plotY > 0) ){
        temp_pos.X = (int)plotX;
        temp_pos.Y = (int)plotY;
        image_bwi(temp_pos, thick_ball_bwi);
      }
    } // End of for loop
    
    // Show the rotation values onscreen
    Display_Debug_Text();
  } // End of while loop
} // End of main


// Empty the balls shape array
void Clear_Balls(){
  int n=0;
  
  // Clear the array
  for(n=0; n<(MAX_BALLS-1); n++){
    bx[n] = 0;
    by[n] = 0;
    bz[n] = 0;
  }
}

// Reset the view values
void Reset_Values(){
  amp = 0.5;
  speed = 0.4;
  freq = 0.6;
  phase = 0;

  ax = 0; 
  ay = 0;
  //az = -3.7;
  az = 0;
  
  axi = 0.00;
  ayi = 0.01;
  azi = 0.02;

  // Clear the screen
  TFT_Fill_Screen(CL_BLACK);
}

// Create a new ball shape
void Generate_Balls(){
  balls=0;
  
  if(shapeNum == 1){
    // Generate a flat plane
    for(x=-4; x<=4; x+=2){
      for(z=-2; z<=3; z+=2){
        bx[balls] = x;
        by[balls] = 2;
        bz[balls] = z;
        balls+=1;
      }
    }
  }else if(shapeNum == 2){
    // Generate a jack shape
    for(yi=-4; yi<=4; yi++){
      for(zi=-4; zi<=4; zi++){
        for(xi=-4; xi<=4; xi++){
          if( ((abs(zi)<1) && (abs(xi)<1)) || ((abs(yi)<1) && (abs(zi)<1)) || ((abs(yi)<1) && (abs(xi)<1)) ) {
            bx[balls] = (float)xi;
            by[balls] = (float)yi;
            bz[balls] = (float)zi;
            balls+=1;
          }
        }
      }
    }
  }else if(shapeNum == 3){
    // Generate a cube shape
    bx[0] = 2;
    by[0] = 2;
    bz[0] = -2;
    
    bx[1] = 2;
    by[1] = 2;
    bz[1] = 2;
    
    bx[2] = -2;
    by[2] = 2;
    bz[2] = 2;

    bx[3] = -2;
    by[3] = 2;
    bz[3] = -2;
    
    bx[4] = 2;
    by[4] = -2;
    bz[4] = -2;
    
    bx[5] = 2;
    by[5] = -2;
    bz[5] = 2;
    
    bx[6] = -2;
    by[6] = -2;
    bz[6] = 2;
    
    bx[7] = -2;
    by[7] = -2;
    bz[7] = -2;
    balls=8;
  }else if(shapeNum == 4){
    // Generate a pyramid shape
    bx[0] = 0;
    by[0] = 1.632;
    bz[0] = 0;
    
    bx[1] = -1.154;
    by[1] = -1.632;
    bz[1] = 2;
    
    bx[2] = -1.154;
    by[2] = -1.632;
    bz[2] = -2;
    
    bx[3] = 2.31;
    by[3] = -1.632;
    bz[3] = 0;
    
    balls=4;
  } else if(shapeNum == 5){
    // Generate an octahedron shape
    bx[0] = 0;
    by[0] = 4;
    bz[0] = 0;
    
    bx[1] = -4;
    by[1] = 0;
    bz[1] = 0;
    
    bx[2] = 0;
    by[2] = 0;
    bz[2] = -4;
    
    bx[3] = 4;
    by[3] = 0;
    bz[3] = 0;
    
    bx[4] = 0;
    by[4] = 0;
    bz[4] = 4;
    
    bx[5] = 0;
    by[5] = -4;
    bz[5] = 0;
    
    balls=6;
  }

  delay_ms(100);
  
  // Toggle the shape used in the next call to Generate_Balls()
  shapeNum += 1;
  if(shapeNum>5) shapeNum = 1;
} // End of Generate_Balls()


// Check for user input
void Check_GS(){
  static int penDown = 0;          // Was the TP screen pressed?
  static unsigned int x_coord = 0, y_coord = 0;   // Touched Coordinates
  static unsigned int prev_x_coord = 0, prev_y_coord = 0;   // Previous Touched Coordinates

  if(leftButton)
  {
    ayi = 0.025;
  }

  if(rightButton)
  {
    ayi = -0.025;
  }

  if(upButton)
  {
    azi = 0.02;
  }

  if(downButton)
  {
    azi = 0;
  }

  // Pause the screen
  if(squareButton)
  {
    delay_ms(5000);
  }

  // Toggle the rotation coordinates display info
  if(circleButton)
  {
    if(displayDebug){
      displayDebug = 0;
    }else{
      displayDebug = 1;
    }
    delay_ms(200);
  }

  if(triangleButton)
  {
    axi = 0.02;
  }

  if(xButton)
  {
    axi = -0.02;
  }

  // The start button will switch the model
  if(startButton)
  {
    //Create the balls shape
    Clear_Balls();
    Generate_Balls();

    // Reset the screen rotation values
    Reset_Values();
  }
  
  // Check for a screen tap to switch the model
  if (TP_TFT_Press_Detect()) {
    // After a PRESS is detected read X-Y and convert it to display dimensions space
    if (TP_TFT_Get_Coordinates(&x_coord, &y_coord) == 0) {
    //Create the balls shape
    Clear_Balls();
    Generate_Balls();

    // Reset the screen rotation values
    Reset_Values();
    delay_ms(150);
    }
  }

}


// Write the current values on screen
void Display_Debug_Text(){
  char axText[7];
  char ayText[7];
  char azText[7];
  char positionText[60];

  // Clear the old text block when the display is toggled
  if( (prevDisplayDebug != displayDebug) && (prevDisplayDebug == 1) ){
    TFT_Set_Pen(CL_BLACK, 1);
    TFT_Set_Brush(1, CL_BLACK, 0, LEFT_TO_RIGHT, CL_BLACK, CL_BLACK);
    TFT_Rectangle(0, 220, 319, 239);
  }

  // Check if the text should be displayed
  if(displayDebug){
    TFT_Set_Brush(1, CL_BLACK, 0, LEFT_TO_RIGHT, CL_BLACK, CL_BLACK);
    TFT_Rectangle(0, 219, 319, 239);

    IntToStr((int)(ax*radToDegrees), axText);
    IntToStr((int)(ay*radToDegrees), ayText);
    IntToStr((int)(az*radToDegrees), azText);

    strcpy(positionText, "Rotation  X:");
    strcat(positionText, axText);
    strcat(positionText, "    Y:");
    strcat(positionText, ayText);
    strcat(positionText, "    Z:");
    strcat(positionText, azText);

    TFT_Write_Text(positionText, 65, 220);
    delay_ms(50);
  }

  // Save the previous value
  prevDisplayDebug = displayDebug;
}